#include "loginwindow.h"
#include "ui_loginwindow.h"

LoginWindow::LoginWindow(MainWindow *parent, bool autologin) :
    QDialog(parent), ui(new Ui::LoginWindow)
{
    ui->setupUi(this);
    myparent = parent;
    servIndexInit = false;

    connect(myparent, SIGNAL(versionMismatch(bool)), this, SLOT(versionMismatch(bool)));
    connect(myparent, SIGNAL(socketCredentialsIncorrect()), this, SLOT(credentialsIncorrect()));
    connect(myparent->socket, SIGNAL(disconnected()), this, SLOT(enableLogin()));
    connect(myparent->socket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(enableLogin()));

    settings = new QSettings(QSettings::UserScope, "Minecraft_GUI", "config");
    int size = settings->beginReadArray("logins");
    myparent->logins.clear();
    for (int i = 0; i < size; i++)
    {
        settings->setArrayIndex(i);
        Server server;
        server.serverName = settings->value("serverName").toString();
        server.serverAddress = settings->value("serverAddress").toString();
        server.serverPort = settings->value("serverPort").toString();
        server.username = settings->value("username").toString();
        server.password = settings->value("password").toString();
        myparent->logins.append(server);
    }
    settings->endArray();

    size = myparent->logins.size();
    for (int i = 0; i < size; i++)
    {
        QString name = myparent->logins.at(i).serverName;
        name.append(" (").append(myparent->logins.at(i).serverAddress).append(")");
        ui->server->insertItem(i, name, QVariant(i));
    }
    int lastSelect = settings->value("lastselect", 0).toInt();
    servIndexInit = true;
    if(size > 0)
    {
        ui->server->setCurrentIndex(lastSelect);
        if(lastSelect == 0)
        {
            on_server_currentIndexChanged(lastSelect);
        }
    }
    bool autoLogin = settings->value("autoLogin", false).toBool();
    ui->autologin->setChecked(autoLogin);
    if (autoLogin && autologin)
    {
        on_Login_clicked();
    }
}

LoginWindow::~LoginWindow()
{
    delete ui;
}

void LoginWindow::on_server_currentIndexChanged(int index)
{
    if(servIndexInit)
    {
        settings->setValue("lastSelect", index);
        ui->userInput->setText(myparent->logins.at(index).username);
        ui->pwdInput->setText(myparent->logins.at(index).password);
    }
}

void LoginWindow::on_Login_clicked()
{
    connect(myparent, SIGNAL(ssocketConnected()), this, SLOT(close()));
    int index = ui->server->currentIndex();
    if (myparent->socketConnected)
    {
        QMessageBox qmbox(QMessageBox::Warning, tr("Already connected"),
                          tr("You are already connected to an other server.<br />If you continue, the current connection will be closed.<br />Continue anyway ?"),
                          QMessageBox::Yes, this, Qt::Dialog);
        qmbox.setIconPixmap(QPixmap(":/images/icones/server_connect"));
        QPushButton *noButton = qmbox.addButton(QMessageBox::No);
        qmbox.exec();
        if (qmbox.clickedButton() == noButton)
        {
            return;
        }
    }
    ui->Login->setEnabled(false);
    myparent->serverID = QString::number(index);
    myparent->socket->abort();
    myparent->socket->connectToHost(myparent->logins.at(index).serverAddress, myparent->logins.at(index).serverPort.toInt());
    if (!myparent->socket->waitForConnected(5000))
    {
        myparent->networkHandler->erreurSocket(myparent->socket->error());
    }
}

void LoginWindow::enableLogin()
{
    ui->Login->setEnabled(true);
}

void LoginWindow::on_autologin_toggled(bool checked)
{
    settings->setValue("autoLogin", checked);
}

void LoginWindow::credentialsIncorrect()
{
    ui->Login->setEnabled(true);
    QMessageBox qmbox(QMessageBox::Warning, tr("Connection refused"),
                      tr("You have been disconnected for the following reason : <br /><br />") + tr("Your suplied credentials are incorrect.<br /><br />"
                                                                                                    "Please verify them. If you are still experiencing trouble, please contact your server administrator."), QMessageBox::Ok, this, Qt::Dialog);
    qmbox.setIconPixmap(QPixmap(":/images/icones/server_error"));
    qmbox.exec();
}

void LoginWindow::versionMismatch(bool type)
{
    ui->Login->setEnabled(true);
    QString message = tr("You have been disconnected for the following reason : <br /><br />") + tr("Your Minecraft GUI version mismatchs<br /><br />");
    if (type)
    {
        message.append(tr("The Minecraft GUI Server you are trying to connect has a higher version than this client.<br /><br />Please go to <a href=\"http://dev.bukkit.org/server-mods/minecraft-gui/\">Dev Bukkit</a> to update your client."));
    }
    else
    {
        message.append(tr("The Minecraft GUI Server you are trying to connect has a lower version than this client.<br /><br />Please get in touch with your server administrator so he could update it."));
    }
    QMessageBox qmbox(QMessageBox::Warning, tr("Connection refused"), message, QMessageBox::Ok, this, Qt::Dialog);
    qmbox.setIconPixmap(QPixmap(":/images/icones/server_error"));
    qmbox.exec();
}

void LoginWindow::on_editServ_clicked()
{
    EditServers editservers(this);
    editservers.exec();
}

Ui::LoginWindow* LoginWindow::getUi()
{
    return ui;
}
