#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <networkhandler.h>

MainWindow::MainWindow(QWidget *parent) :
	QMainWindow(parent), ui(new Ui::MainWindow)
{
	ui->setupUi(this);

        version = 0.2;
	socket = new QTcpSocket(this);
	socketConnected = false;
	credentialsSent = false;
	networkHandler = new NetworkHandler(this);

	connect(networkHandler, SIGNAL(versionMismatch(bool)), this, SLOT(versionMismatch1(bool)));
	connect(networkHandler, SIGNAL(ssocketConnected()), this, SLOT(ssocketConnected1()));
	connect(networkHandler, SIGNAL(socketCredentialsIncorrect()), this, SLOT(socketCredentialsIncorrect1()));
}

MainWindow::~MainWindow()
{
	delete ui;
}

Ui::MainWindow* MainWindow::getUi()
{
	return ui;
}

void MainWindow::on_actionObtenir_de_l_aide_triggered()
{
    QDesktopServices::openUrl(QUrl("http://dev.bukkit.org/server-mods/minecraft-gui/"));
}

void MainWindow::on_action_propos_triggered()
{
    QMessageBox::about(this, tr("About Minecraft GUI"), tr("<strong>Author :</strong> Mika56<br /><strong>Version :</strong> %1<br /><strong>Licence :</strong> <a href=\"http://dev.bukkit.org/licenses/14-apache-license-version-2-0/\">Apache License version 2.0</a><br /><br />Images from <a href=\"http://www.fatcow.com/free-icons\">Fatcow.com</a><br />Minecraft is a trademark of Mojang.", "Text from the About dialog. Please don't remove my name. You can add yours by adding a Translator line").arg(version));
}

void MainWindow::on_action_propos_de_Qt_triggered()
{
	QMessageBox::aboutQt(this);
}

void MainWindow::on_actionQuitter_triggered()
{
	QApplication::quit();
}

void MainWindow::on_send_clicked()
{
	networkHandler->sendChat(ui->chatText->text());
	ui->chatText->clear();
}

void MainWindow::versionMismatch1(bool type)
{
	emit versionMismatch(type);
}
void MainWindow::ssocketConnected1()
{
	emit ssocketConnected();
}
void MainWindow::socketCredentialsIncorrect1()
{
	emit socketCredentialsIncorrect();
}

void MainWindow::on_reloadButton_clicked()
{
    networkHandler->sendSocket("RELOAD");
}

void MainWindow::on_stopButton_clicked()
{
    networkHandler->sendSocket("STOP SERVER");
}

void MainWindow::on_actionConnect_triggered()
{
    LoginWindow LoginWindow(this, false);
    LoginWindow.exec();
}

void MainWindow::on_joinButton_clicked()
{
    QSettings* settings = new QSettings(QSettings::UserScope, "Minecraft_GUI", "config");
    QString exeLoc = settings->value("mc_exe", "").toString();
    QStringList exeArgs;
    if(exeLoc.isEmpty() || !QFile::exists(exeLoc))
    {
        QMessageBox qmbox(QMessageBox::Warning, tr("Unable to join server"), tr("Minecraft's executable not found or not defined"), QMessageBox::Ok, this, Qt::Dialog);
        qmbox.setIconPixmap(QPixmap(":/images/icones/server_error"));
        qmbox.exec();
        return;
    }
    if(exeLoc.endsWith(".jar"))
    {
        exeArgs << "-cp" << exeLoc << "net.minecraft.LauncherFrame";
        exeLoc = "java";
    }
    bool ok;
    QString pass = QInputDialog::getText(this, tr("Enter your Minecraft's password"), tr("Password"), QLineEdit::Password, "", &ok);
    if(ok && !pass.isEmpty())
    {
        exeArgs << networkHandler->currentUsername << pass << networkHandler->currentServerLogin;
        process.start(exeLoc, exeArgs);
    }
}

void MainWindow::on_actionSettings_triggered()
{
    Preferences Preferences(this);
    Preferences.exec();
}
