#ifndef NETWORKHANDLER_H
#define NETWORKHANDLER_H

#include <QObject>
#include <QtNetwork>
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "serverStruct.h"

class MainWindow;
class NetworkHandler : public QObject
{
	Q_OBJECT

public:
    NetworkHandler(MainWindow *parent);
    void sendSocket(QString message);
    void sendChat(QString message);
    void setTestServer(const Server &server);
    void endTestServer();
    void disconnectFromHost();
    QString currentUsername;
    QString currentServerLogin;

private:
    MainWindow *myParent;
    QString packetSeparator;
    int storedMaxPlayers;
    int storedOnlinePlayers;
    bool isTest;
    Server testServer;

    void processPaquet(QString paquet);
    void processRights(QString paquet);
    void processBasicInfos(QString paquet);
    void processServerInfos(QString paquet);
    void processUserCount(int onlinePlayers, int maxPlayers);
    void processUserCount(int onlinelayers);
    void processChatMsg(QString paquet);
    void processJoin(QString paquet);
    void processQuit(QString paquet);

public slots:
    void erreurSocket(QAbstractSocket::SocketError erreur);

private slots:
	void connected();
	void disconnected();
        void datasReceived();

signals:
	void versionMismatch(bool);
	void ssocketConnected();
	void socketCredentialsIncorrect();
};

#endif // NETWORKHANDLER_H
