#include "preferences.h"
#include "ui_preferences.h"

Preferences::Preferences(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Preferences)
{
    ui->setupUi(this);

    ui->localeList->addItem("English", "en");
    ui->localeList->addItem("Franais", "fr");

    settings = new QSettings(QSettings::UserScope, "Minecraft_GUI", "config");
    QString exeLoc = settings->value("mc_exe", "").toString();
    QString locale = settings->value("locale", "").toString();
    bool autoStart = settings->value("autostart", false).toBool();
    previousLocale = locale;
    ui->autoStart->setChecked(autoStart);
    ui->mc_exe_loc->setText(exeLoc);
    if(!locale.isEmpty())
    {
        ui->localeList->setCurrentIndex(ui->localeList->findData(locale, int(Qt::UserRole)));
    }
}

Preferences::~Preferences()
{
    settings->setValue("mc_exe", ui->mc_exe_loc->text());
    QString newLocale = ui->localeList->itemData(ui->localeList->currentIndex()).toString();
    if(newLocale != previousLocale)
    {
        QMessageBox::information(this, tr("Locale changed"), tr("You changed Minecraft GUI's locale.<br />Please restart it to apply new locale."));
    }
    settings->setValue("locale", newLocale);
    if(ui->autoStart->isChecked())
    {
#ifndef Q_WS_WIN
        QMessageBox::information(this, tr("Autostart unavailable"), tr("Minecraft GUI can only start automaticaly on Windows for the moment.<br />Please manualy add a shortcut to the Minecraft GUI's executable."));
#else
        settings->setValue("autostart", true);
        QSettings *start = new QSettings("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run", QSettings::NativeFormat);
        start->setValue("MinecraftGUI", QCoreApplication::applicationFilePath());

#endif
    }
    else
    {
        QSettings *start = new QSettings("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run", QSettings::NativeFormat);
        settings->setValue("autostart", false);
        start->remove("MinecraftGUI");
    }
    delete ui;
}

void Preferences::on_mc_exe_loc_find_clicked()
{
    ui->mc_exe_loc->setText(QFileDialog::getOpenFileName(this, tr("Define Minecraft executable"), "", tr("Minecraft executable (*.exe *.jar)")));
}
