package me.Mika56.McGui;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Vector;

public class SocketServer implements Runnable
{
	protected int serverPort = 62484;
	protected McGui plugin = null;
	protected ServerSocket serverSocket = null;
	protected boolean isStopped = true;
	protected Thread runningThread = null;
	protected Vector<WorkerRunnable> _tabClients = new Vector<WorkerRunnable>();
	protected int _nbClients = 0;
	
	public SocketServer(int port, McGui plugin)
	{
		this.serverPort = port;
		this.plugin = plugin;
		start();
	}

	public void run()
	{
		synchronized(this)
		{
			this.runningThread = Thread.currentThread();
			isStopped = false;
		}
		try
		{
			this.serverSocket = new ServerSocket(this.serverPort);
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
		finally
		{
			while(!isStopped())
			{
				Socket clientSocket = null;
				try
				{
					clientSocket = this.serverSocket.accept();
					clientSocket.setTcpNoDelay(false);
				}
				catch(IOException e)
				{
					if(isStopped())
					{
						return;
					}
					e.printStackTrace();
				}
				new WorkerRunnable(clientSocket, this);
			}
		}
	}
	
	private synchronized boolean isStopped()
	{
		return this.isStopped;
	}

	public synchronized void stop()
	{
		this.isStopped = true;
		try
		{
			this.serverSocket.close();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
	}
	
	public synchronized void start()
	{
		(new Thread(this)).start();
	}

	synchronized public int addClient(WorkerRunnable worker)
	{
		this._nbClients++;
		this._tabClients.addElement(worker);
		return _tabClients.size()-1;
	}
	synchronized public void sendAll(String message)
	{
		WorkerRunnable worker;
		for(int i=0;i<this._tabClients.size(); i++)
		{
			worker = (WorkerRunnable) _tabClients.elementAt(i);
			if(worker.isLogged)
			{
				worker.sendPacket(message);
			}
		}
	}
	synchronized public Vector<WorkerRunnable> getAllClients()
	{
		return this._tabClients;
	}

	synchronized public void delClient(int i)
	{
		this._nbClients--;
		if(this._tabClients.elementAt(i) != null)
		{
			this._tabClients.removeElementAt(i);
		}
	}
}
